@@ -16,10 +16,10 @@ from inspect import stack
 from os import environ as os_environ
 from string import ascii_lowercase
 from boto3 import resource
-from boto3.dynamodb.conditions import Key, And, ConditionExpressionBuilder
+from boto3.dynamodb.conditions import Key, And, ConditionExpressionBuilder, ConditionBase
 from botocore.exceptions import ClientError
 from copy import deepcopy
-from typing import Iterable
+from typing import Iterable, List
 
 _ddb_resource = resource("dynamodb", **{"region_name": os_environ["AWS_REGION"] if "AWS_REGION" in os_environ else "us-east-1"})
 
@@ -64,6 +64,15 @@ def _cast_table_name(table_name: str) -> str:
     return "-".join(name_components)
 
 
+def _combine_and_conditions(conditions: list):
+    if len(conditions) == 1:
+        con = conditions[0]
+
+    else:
+        con = And(*conditions)
+    return ConditionExpressionBuilder().build_expression(con)
+
+
 class Table:
     def __init__(self, table_name, special_resource_config: dict = False):
         self.__table_name = table_name
@@ -205,12 +214,17 @@ class Table:
         elif len(index_primary) > len(self.indexes[index]):
             self.custom_exception.wrong_primary_key(index_primary)
 
-    def _create_projection_expression(self, attributes_to_get):
+    def _create_projection_expression(self, attributes_to_get, existing_data: dict = None):
+        if existing_data is None:
+            existing_data = dict()
+
         letter_count = 0
         attribute_expression_map = dict()
         attribute_expressions = list()
 
         def next_expression(attribute_name, lc):
+            if attribute_name in existing_data.get("ExpressionAttributeNames", dict()).values():
+                return {va: k for k, va in existing_data.get("ExpressionAttributeNames", dict()).items()}[attribute_name], lc
             letter = f"#{_value_update_chars[lc].upper()}"
             attribute_expression_map[letter] = attribute_name
             lc += 1
@@ -253,7 +267,7 @@ class Table:
         ----------
         attributes_to_get: list, optional
             specify certain attributes to get
-        primary_dict: dict
+        primary_dict: str
             primary keys for the item to retrieve
 
         Returns
@@ -690,7 +704,7 @@ class Table:
 
     def update_add_set(
         self,
-        new_data,
+        new_data: dict,
         set_new_attribute_if_not_existent=False,
         create_item_if_non_existent=False,
         returns: UpdateReturns = UpdateReturns.NONE,
@@ -790,18 +804,23 @@ class Table:
     def remove_from_set(
             self,
             path_to_set: list,
-            items_to_delete: (list, set),
+            items_to_delete: (List[set], set),
             returns: UpdateReturns = UpdateReturns.NONE,
             condition=None,
             **primary_dict,
     ):
         if not isinstance(path_to_set[0], list):
             path_to_set = [path_to_set]
+        if not isinstance(items_to_delete, list):
+            items_to_delete = [items_to_delete]
+        for i, s in enumerate(items_to_delete):
+            if not isinstance(s, set):
+                items_to_delete[i] = set(s)
         return self.__general_update(
             require_attributes_already_present=True,
             create_item_if_non_existent=False,
             remove_data=path_to_set.copy(),
-            remove_set_item=items_to_delete if isinstance(items_to_delete, list) else [items_to_delete],
+            remove_set_item=items_to_delete,
             returns=returns,
             direct_condition=condition,
             **primary_dict
@@ -839,12 +858,96 @@ class Table:
             for item in self.scan()["Items"]:
                 batch.delete_item(Key={key: item[key] for key in self.pk})
 
-    def query(self, **query_data):
+    def query(
+            self,
+            attributes_to_get: list = None,
+            max_results: int = None,
+            offset_last_key: (str, int, float, dict) = None,
+            range_condition: ConditionBase = None,
+            index: str = None,
+            **query_keys
+    ) -> dict:
+        """
+        query data based
+
+        Parameters
+        ----------
+        attributes_to_get: list
+            specify the attributes to return (for decreasing transferred data amount)
+        max_results: int, optional
+            limit the number of items to return
+        offset_last_key: str, int, float, dict, optional
+            for pagination: if many values provide the last key that shall not be included
+        range_condition: ConditionBase, optional
+            specify conditions for the range key to match
+        index: str, optional
+            if query should be executed on index
+        query_keys: str
+            primary or index keys
+
+        Returns
+        -------
+        dict
+            Count: int, ScannedCount: int, Items: list, (LastEvaluatedKey: (str, int, float))
+            items returned, number scanned for operation, actual items, last range key returned (for pagination)
+
+        """
+        query_data = dict()
+        range_key = None
+
+        if not index:
+            query_keys = self._cast_primary_keys(query_keys)
+            primary_key = self.pk[0]
+            if len(self.pk) > 1:
+                range_key = self.pk[1]
+        else:
+            if offset_last_key is not None and not isinstance(offset_last_key, dict):
+                raise TypeError(
+                    {
+                        "statusCode": 400,
+                        "body": "querying on index requires dictionary with last evaluated values",
+                        "headers": {"Content-Type": "text/plain"},
+                    }
+                )
+            query_keys = self._cast_index_keys(index, query_keys)
+            query_data["IndexName"] = index
+            primary_key = self.indexes[index][0]
+            if len(self.indexes[index]) > 1:
+                range_key = self.indexes[index][1]
+
+        condition_expression_array = [Key(k).eq(v) for k, v in query_keys.items()]
+        if range_condition:
+            condition_expression_array.append(type(range_condition)(Key(range_key), *range_condition._values))
+
+        query_data["KeyConditionExpression"], query_data["ExpressionAttributeNames"], query_data["ExpressionAttributeValues"] = _combine_and_conditions(condition_expression_array)
+
+        if attributes_to_get:
+            expression, name_map = self._create_projection_expression(attributes_to_get, query_data)
+            query_data["ProjectionExpression"] = expression
+            query_data["ExpressionAttributeNames"].update(name_map)
+        else:
+            query_data["Select"] = SelectReturns.ALL_ATTRIBUTES
+
+        if max_results:
+            query_data["Limit"] = max_results
+        if offset_last_key:
+            if not index:
+                start_key = {primary_key: query_keys[primary_key], range_key: offset_last_key}
+            else:
+                start_key = object_with_float_to_decimal(offset_last_key)
+            query_data["ExclusiveStartKey"] = start_key
+
         response = self.__table.query(
             **query_data
         )
         if items := response.get("Items", list()):
             response["Items"] = object_with_decimal_to_float(items)
+        if "LastEvaluatedKey" in response:
+            response["LastEvaluatedKey"] = object_with_decimal_to_float(response["LastEvaluatedKey"])
+            if not index:
+                response["LastEvaluatedKey"] = response["LastEvaluatedKey"][range_key]
+            else:
+                response["LastEvaluatedKey"] = response["LastEvaluatedKey"]
         return response
 
     def __return_dict_of_pk_items_from_multiple_item_response(self, object_list: list, convert: bool) -> (dict, list):
@@ -884,19 +987,10 @@ class Table:
         """
         index_keys = self._cast_index_keys(index, index_keys)
         self._index_key_checker(index, index_keys)
-        expression_array = [Key(k).eq(v) for k, v in index_keys.items()]
-        key_condition_expression = expression_array[0] if len(expression_array) == 1 else And(*expression_array)
-        query_data = {
-            "IndexName": index,
-            "Select": SelectReturns.ALL_ATTRIBUTES,
-            "KeyConditionExpression": key_condition_expression
-        }
-        if attributes_to_get:
-            expression, name_map = self._create_projection_expression(attributes_to_get)
-            query_data.update({"ProjectionExpression": expression, "ExpressionAttributeNames": name_map})
-            query_data.pop("Select")
         object_list = self.query(
-            **query_data
+            index=index,
+            attributes_to_get=attributes_to_get,
+            **index_keys
         )["Items"]
         if len(object_list) == 0:
             raise FileNotFoundError
